"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const querystring_1 = __importDefault(require("querystring"));
const json5_1 = __importDefault(require("json5"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const rollup_pluginutils_1 = require("rollup-pluginutils");
const vue_i18n_1 = require("vue-i18n");
const debug_1 = require("debug");
const debug = debug_1.debug('rollup-plugin-vue-i18n');
// eslint-disable-next-line @typescript-eslint/no-unused-vars
function i18n(options) {
    const filter = rollup_pluginutils_1.createFilter([/vue&type=i18n/]);
    return {
        name: 'rollup-plugin-vue-i18n',
        transform(source, id) {
            debug('transform id', id);
            if (filter(id)) {
                const query = parseVuePartRequest(id);
                const variableName = query.global ? '__i18nGlobal' : '__i18n';
                return {
                    code: `export default function i18n(Component) {\n` +
                        `  const options = typeof Component === 'function' ? Component.options : Component\n` +
                        `  options.${variableName} = options.${variableName} || []\n` +
                        `  options.${variableName}.push(${stringify(parse(source, query), query)})\n` +
                        `}`,
                    map: {
                        mappings: ''
                    }
                };
            }
        }
    };
}
exports.default = i18n;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function stringify(data, query) {
    const { locale } = query;
    if (locale) {
        return vue_i18n_1.friendlyJSONstringify(Object.assign({}, { [locale]: data }));
    }
    else {
        return vue_i18n_1.friendlyJSONstringify(data);
    }
}
function parse(source, query) {
    const value = source.trim();
    const { lang } = query;
    switch (lang) {
        case 'yaml':
        case 'yml':
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            return js_yaml_1.default.safeLoad(value);
        case 'json5':
            return json5_1.default.parse(value);
        default:
            return JSON.parse(value);
    }
}
function parseVuePartRequest(id) {
    const [filename, query] = id.split('?', 2);
    if (!query) {
        return { vue: false, filename };
    }
    const raw = querystring_1.default.parse(query);
    if ('vue' in raw) {
        const langPart = Object.keys(raw).find(key => /lang\./i.test(key));
        const part = {
            ...raw,
            filename,
            vue: true,
            index: Number(raw.index),
            src: 'src' in raw,
            scoped: 'scoped' in raw
        };
        if (langPart) {
            const [, lang] = langPart.split('.');
            part.lang = lang;
        }
        return part;
    }
    return { vue: false, filename };
}
